@@ -6,6 +6,7 @@ of galaxy images.
 # Licensed under a 3-Clause BSD License.
 from __future__ import absolute_import, division, print_function
 
+from pkg_resources import parse_version
 import warnings
 import time
 import numpy as np
@@ -15,7 +16,7 @@ import scipy.signal
 import skimage.measure
 import skimage.transform
 import skimage.feature
-import skimage.morphology
+import skimage.segmentation
 from astropy.utils import lazyproperty
 from astropy.stats import sigma_clipped_stats
 from astropy.modeling import models, fitting
@@ -25,7 +26,7 @@ import photutils
 __all__ = ['ConvolvedSersic2D', 'SourceMorphology', 'source_morphology',
            '__version__']
 
-__version__ = '0.3.3'
+__version__ = '0.3.6'
 
 def _quantile(sorted_values, q):
     """
@@ -62,12 +63,12 @@ def _local_variance(image):
         [1, 0, 1],
         [1, 1, 1]], dtype=np.float64)
     w = w / np.sum(w)
-    
+
     # Use the fact that var(x) = <x^2> - <x>^2.
     local_mean = ndi.convolve(image, w)
     local_mean2 = ndi.convolve(image**2, w)
     local_var = local_mean2 - local_mean**2
-    
+
     return local_var
 
 def _aperture_area(ap, mask, **kwargs):
@@ -86,7 +87,10 @@ def _aperture_mean_nomask(ap, image, **kwargs):
     This avoids problems when the aperture is larger than the
     region of interest.
     """
-    return ap.do_photometry(image, **kwargs)[0][0] / ap.area()
+    if parse_version(photutils.__version__) <= parse_version('0.6'):
+        return ap.do_photometry(image, **kwargs)[0][0] / ap.area()
+    else:
+        return ap.do_photometry(image, **kwargs)[0][0] / ap.area
 
 def _fraction_of_total_function_circ(r, image, center, fraction, total_sum):
     """
@@ -157,11 +161,11 @@ def _fraction_of_total_function_ellip(a, image, center, elongation, theta,
         cur_fraction = 0.0
     else:
         b = a / elongation
-        ap = photutils.EllipticalAperture(center, a, b, theta)
+        ap = photutils.EllipticalAperture(center, a, b, theta=theta)
         # Force flux sum to be positive:
         ap_sum = np.abs(ap.do_photometry(image, method='exact')[0][0])
         cur_fraction = ap_sum / total_sum
-    
+
     return cur_fraction - fraction
 
 def _radius_at_fraction_of_total_ellip(image, center, elongation, theta,
@@ -174,7 +178,8 @@ def _radius_at_fraction_of_total_ellip(image, center, elongation, theta,
     flag = 0  # flag=1 indicates a problem
 
     b_total = a_total / elongation
-    ap_total = photutils.EllipticalAperture(center, a_total, b_total, theta)
+    ap_total = photutils.EllipticalAperture(
+        center, a_total, b_total, theta=theta)
 
     total_sum = ap_total.do_photometry(image, method='exact')[0][0]
     if total_sum == 0:
@@ -196,7 +201,7 @@ def _radius_at_fraction_of_total_ellip(image, center, elongation, theta,
             a, image, center, elongation, theta, fraction, total_sum)
         if curval == 0:
             warnings.warn('Found root by pure chance!', AstropyUserWarning)
-            return r, flag
+            return a, flag
         elif curval < 0:
             a_min = a
         elif curval > 0:
@@ -239,7 +244,9 @@ class ConvolvedSersic2D(models.Sersic2D):
         if cls.psf is None:
             raise Exception('Must specify PSF using set_psf method.')
 
-        return scipy.signal.fftconvolve(z_sersic, cls.psf, mode='same')
+        # Apparently, scipy.signal also wants double:
+        return scipy.signal.fftconvolve(
+            np.float64(z_sersic), np.float64(cls.psf), mode='same')
 
 
 class SourceMorphology(object):
@@ -253,7 +260,7 @@ class SourceMorphology(object):
         A 2D image containing the sources of interest.
         The image must already be background-subtracted.
     segmap : array-like (int) or ``photutils.SegmentationImage``
-        A 2D segmentation map where different sources are 
+        A 2D segmentation map where different sources are
         labeled with different positive integer values.
         A value of zero is reserved for the background.
         It is assumed that the sum of pixel values within each
@@ -292,7 +299,7 @@ class SourceMorphology(object):
     n_sigma_outlier : scalar, optional
         The number of standard deviations that define a pixel as an
         outlier, relative to its 8 neighbors. Outlying pixels are
-        removed as described in Lotz et al. (2004) If the value is zero
+        removed as described in Lotz et al. (2004). If the value is zero
         or negative, outliers are not removed. The default value is 10.
     annulus_width : float, optional
         The width (in pixels) of the annuli used to calculate the
@@ -359,7 +366,7 @@ class SourceMorphology(object):
                  petro_extent_cas=1.5, petro_fraction_cas=0.25,
                  boxcar_size_mid=3.0, niter_bh_mid=5, sigma_mid=1.0,
                  petro_extent_flux=2.0, boxcar_size_shape_asym=3.0,
-                 sersic_maxiter=500, segmap_overlap_ratio=0.5):
+                 sersic_maxiter=500, segmap_overlap_ratio=0.25):
         self._image = image
         self._segmap = segmap
         self.label = label
@@ -384,110 +391,8 @@ class SourceMorphology(object):
         self._sersic_maxiter = sersic_maxiter
         self._segmap_overlap_ratio = segmap_overlap_ratio
 
-        # Measure runtime
-        start = time.time()
-
-        if not isinstance(self._segmap, photutils.SegmentationImage):
-            self._segmap = photutils.SegmentationImage(self._segmap)
-
-        # Check sanity of input data
-        if float(photutils.__version__) < 0.5:
-            self._segmap.check_label(self.label)
-        else:
-            self._segmap.check_labels([self.label])
-        assert self._segmap.data.shape == self._image.shape
-        if self._mask is not None:
-            assert self._mask.shape == self._image.shape
-            assert self._mask.dtype == np.bool8
-        if self._weightmap is not None:
-            assert self._weightmap.shape == self._image.shape
-
-        # Normalize PSF
-        if self._psf is not None:
-            self._psf = self._psf / np.sum(self._psf)
-
-        # Check that the labeled galaxy segment has a positive flux sum:
-        assert np.sum(self._cutout_stamp_maskzeroed_no_bg) > 0
-
-        # These flags will be modified during the calculations:
-        self.flag = 0  # attempts to flag bad measurements
-        self.flag_sersic = 0  # attempts to flag bad Sersic fits
-
-        # If something goes wrong, use centroid instead of asymmetry center
-        # (better performance in some pathological cases, e.g. GOODS-S 32143):
-        self._use_centroid = False
-
-        # Centroid of the source relative to the "postage stamp" cutout:
-        self._xc_stamp = self.xc_centroid - self.xmin_stamp
-        self._yc_stamp = self.yc_centroid - self.ymin_stamp
-
-        # Print warning if centroid is masked:
-        ic, jc = int(self._yc_stamp), int(self._xc_stamp)
-        if self._cutout_stamp_maskzeroed[ic, jc] == 0:
-            warnings.warn('Centroid is masked.', AstropyUserWarning)
-            self.flag = 1
-
-        # Position of the source's brightest pixel relative to the stamp cutout:
-        maxval = np.max(self._cutout_stamp_maskzeroed_no_bg)
-        maxval_stamp_pos = np.argwhere(self._cutout_stamp_maskzeroed_no_bg == maxval)[0]
-        self._x_maxval_stamp = maxval_stamp_pos[1]
-        self._y_maxval_stamp = maxval_stamp_pos[0]
-
-        # ------------------------------------------------------------------
-        # NOTE: no morphology calculations have been done so far, but
-        # this will change below this line. Modify this __init__ with care.
-        # ------------------------------------------------------------------
-
-        # For simplicity, evaluate all "lazy" properties at once:
-        self._calculate_morphology()
-
-        # Check if image is background-subtracted; set flag=1 if not.
-        if np.abs(self.sky_mean) > self.sky_sigma:
-            warnings.warn('Image is not background-subtracted.', AstropyUserWarning)
-            self.flag = 1
-
-        # Check segmaps and set flag=1 if they are very different
-        self._check_segmaps()
-
-        # Save runtime
-        self.runtime = time.time() - start
-
-    def __getitem__(self, key):
-        return getattr(self, key)
-
-    def _get_badpixels(self, image):
-        """
-        Detect outliers (bad pixels) as described in Lotz et al. (2004).
-
-        Notes
-        -----
-        ndi.generic_filter(image, np.std, ...) is too slow,
-        so we do a workaround using ndi.convolve.
-        """
-        # Pixel weights, excluding central pixel.
-        w = np.array([
-            [1, 1, 1],
-            [1, 0, 1],
-            [1, 1, 1]], dtype=np.float64)
-        w = w / np.sum(w)
-        
-        # Use the fact that var(x) = <x^2> - <x>^2.
-        local_mean = ndi.convolve(image, w)
-        local_mean2 = ndi.convolve(image**2, w)
-        local_std = np.sqrt(local_mean2 - local_mean**2)
-
-        # Get "bad pixels"
-        badpixels = (np.abs(image - local_mean) >
-                      self._n_sigma_outlier * local_std)
-        
-        return badpixels
-
-    def _calculate_morphology(self):
-        """
-        Calculate all morphological parameters, which are stored
-        as "lazy" properties.
-        """
-        quantities = [
+        # A list of all the properties that will be calculated:
+        self._quantity_names = [
             'xc_centroid',
             'yc_centroid',
             'ellipticity_centroid',
@@ -539,7 +444,134 @@ class SourceMorphology(object):
             'nx_stamp',
             'ny_stamp',
         ]
-        for q in quantities:
+
+        # Measure runtime
+        start = time.time()
+
+        if not isinstance(self._segmap, photutils.SegmentationImage):
+            self._segmap = photutils.SegmentationImage(self._segmap)
+
+        # Check sanity of input data
+        if parse_version(photutils.__version__) < parse_version('0.5'):
+            self._segmap.check_label(self.label)
+        else:
+            self._segmap.check_labels([self.label])
+        assert self._segmap.data.shape == self._image.shape
+        if self._mask is not None:
+            assert self._mask.shape == self._image.shape
+            assert self._mask.dtype == np.bool8
+        if self._weightmap is not None:
+            assert self._weightmap.shape == self._image.shape
+
+        # Normalize PSF
+        if self._psf is not None:
+            self._psf = self._psf / np.sum(self._psf)
+
+        # These flags will be modified during the calculations:
+        self.flag = 0  # attempts to flag bad measurements
+        self.flag_sersic = 0  # attempts to flag bad Sersic fits
+        self.flag_catastrophic = 0  # this one is reserved for really bad cases
+
+        # If something goes wrong, use centroid instead of asymmetry center
+        # (better performance in some pathological cases, e.g. GOODS-S 32143):
+        self._use_centroid = False
+
+        # Check that the labeled galaxy segment has a positive flux sum.
+        # If not, this is bad enough to abort all calculations and return
+        # an empty object.
+        if np.sum(self._cutout_stamp_maskzeroed_no_bg) <= 0:
+            warnings.warn('Total flux is nonpositive. Returning empty object.',
+                          AstropyUserWarning)
+            self._abort_calculations()
+            return
+
+        # Centroid of the source relative to the "postage stamp" cutout:
+        self._xc_stamp = self.xc_centroid - self.xmin_stamp
+        self._yc_stamp = self.yc_centroid - self.ymin_stamp
+
+        # Print warning if centroid is masked:
+        ic, jc = int(self._yc_stamp), int(self._xc_stamp)
+        if self._cutout_stamp_maskzeroed[ic, jc] == 0:
+            warnings.warn('Centroid is masked.', AstropyUserWarning)
+            self.flag = 1
+
+        # Position of the source's brightest pixel relative to the stamp cutout:
+        maxval = np.max(self._cutout_stamp_maskzeroed_no_bg)
+        maxval_stamp_pos = np.argwhere(self._cutout_stamp_maskzeroed_no_bg == maxval)[0]
+        self._x_maxval_stamp = maxval_stamp_pos[1]
+        self._y_maxval_stamp = maxval_stamp_pos[0]
+
+        # --------------------------------------------------------------------
+        # NOTE: most morphology calculations have not been performed yet, but
+        # this will change below this line. Modify this __init__ with care.
+        # --------------------------------------------------------------------
+
+        # For simplicity, evaluate all "lazy" properties at once:
+        self._calculate_morphology()
+
+        # Check if image is background-subtracted; set flag=1 if not.
+        if np.abs(self.sky_mean) > self.sky_sigma:
+            warnings.warn('Image is not background-subtracted.', AstropyUserWarning)
+            self.flag = 1
+
+        # Check segmaps and set flag=1 if they are very different
+        self._check_segmaps()
+
+        # Save runtime
+        self.runtime = time.time() - start
+
+    def __getitem__(self, key):
+        return getattr(self, key)
+
+    def _get_badpixels(self, image):
+        """
+        Detect outliers (bad pixels) as described in Lotz et al. (2004).
+
+        Notes
+        -----
+        ndi.generic_filter(image, np.std, ...) is too slow,
+        so we do a workaround using ndi.convolve.
+        """
+        # Pixel weights, excluding central pixel.
+        w = np.array([
+            [1, 1, 1],
+            [1, 0, 1],
+            [1, 1, 1]], dtype=np.float64)
+        w = w / np.sum(w)
+
+        # Use the fact that var(x) = <x^2> - <x>^2.
+        local_mean = ndi.convolve(image, w)
+        local_mean2 = ndi.convolve(image**2, w)
+        local_std = np.sqrt(local_mean2 - local_mean**2)
+
+        # Get "bad pixels"
+        badpixels = (np.abs(image - local_mean) >
+                     self._n_sigma_outlier * local_std)
+
+        return badpixels
+
+    def _abort_calculations(self):
+        """
+        Some cases are so bad that basically nothing can be measured
+        (e.g. a bunch of pixels with a nonpositive total sum). We
+        deal with these cases by creating an "empty" object and
+        interrupting the constructor.
+        """
+        for q in self._quantity_names:
+            setattr(self, q, -99.0)
+        self.nx_stamp = -99
+        self.ny_stamp = -99
+        self.flag = 1
+        self.flag_sersic = 1
+        self.flag_catastrophic = 1
+        self.runtime = -99.0
+
+    def _calculate_morphology(self):
+        """
+        Calculate all morphological parameters, which are stored
+        as "lazy" properties.
+        """
+        for q in self._quantity_names:
             tmp = self[q]
 
     def _check_segmaps(self):
@@ -576,6 +608,14 @@ class SourceMorphology(object):
         yc = M[1, 0] / M[0, 0]
         xc = M[0, 1] / M[0, 0]
 
+        ny, nx = self._cutout_stamp_maskzeroed_no_bg.shape
+        if (yc < 0) or (yc >= ny) or (xc < 0) or (xc >= nx):
+            warnings.warn('Centroid is out-of-range. Fixing at center of ' +
+                          'postage stamp (bad!).', AstropyUserWarning)
+            yc = ny / 2.0
+            xc = nx / 2.0
+            self.flag = 1
+
         return np.array([xc, yc])
 
     @lazyproperty
@@ -648,7 +688,8 @@ class SourceMorphology(object):
 
         return eigvals
 
-    def _ellipticity_generic(self, eigvals):
+    @staticmethod
+    def _ellipticity_generic(eigvals):
         """
         The ellipticity of (the Gaussian function that has the same
         second-order moments as) the source. Note that we allow
@@ -659,7 +700,8 @@ class SourceMorphology(object):
 
         return 1.0 - (b / a)
 
-    def _elongation_generic(self, eigvals):
+    @staticmethod
+    def _elongation_generic(eigvals):
         """
         The elongation of (the Gaussian function that has the same
         second-order moments as) the source. Note that we allow
@@ -670,15 +712,16 @@ class SourceMorphology(object):
 
         return a / b
 
-    def _orientation_generic(self, covariance):
+    @staticmethod
+    def _orientation_generic(covariance):
         """
         The orientation (in radians) of the source.
         """
         x2, xy, xy, y2 = covariance.flat
-        
+
         # SExtractor manual, eq. (21):
         theta = 0.5 * np.arctan2(2.0 * xy, x2 - y2)
-        
+
         return theta
 
     @lazyproperty
@@ -733,7 +776,7 @@ class SourceMorphology(object):
         assert self._cutout_extent >= 1.0
         assert self._min_cutout_size >= 2
         # Get dimensions of original bounding box
-        s = self._segmap.slices[self.label - 1]
+        s = self._segmap.slices[self._segmap.get_index(self.label)]
         xmin, xmax = s[1].start, s[1].stop - 1
         ymin, ymax = s[0].start, s[0].stop - 1
         dx, dy = xmax + 1 - xmin, ymax + 1 - ymin
@@ -843,7 +886,7 @@ class SourceMorphology(object):
         by ``_slice_stamp``. Pixels belonging to other sources
         (as well as pixels where ``mask`` == 1) are set to zero,
         but the background is left alone.
-        
+
         In addition, NaN or inf values are removed at this point,
         as well as badpixels (outliers).
         """
@@ -917,7 +960,7 @@ class SourceMorphology(object):
     def _petrosian_function_circ(self, r, center):
         """
         Helper function to calculate the circular Petrosian radius.
-        
+
         For a given radius ``r``, return the ratio of the mean flux
         over a circular annulus divided by the mean flux within the
         circle, minus "eta" (eq. 4 from Lotz et al. 2004). The root
@@ -936,7 +979,7 @@ class SourceMorphology(object):
             circ_annulus, image, method='exact'))
         circ_aperture_mean_flux = np.abs(_aperture_mean_nomask(
             circ_aperture, image, method='exact'))
-        
+
         if circ_aperture_mean_flux == 0:
             warnings.warn('[rpetro_circ] Mean flux is zero.', AstropyUserWarning)
             ratio = 1.0
@@ -991,7 +1034,7 @@ class SourceMorphology(object):
                     break
             r += dr
 
-        rpetro_circ = opt.brentq(self._petrosian_function_circ, 
+        rpetro_circ = opt.brentq(self._petrosian_function_circ,
                                  r_min, r_max, args=(center,), xtol=1e-6)
 
         return rpetro_circ
@@ -1031,13 +1074,13 @@ class SourceMorphology(object):
     def _petrosian_function_ellip(self, a, center, elongation, theta):
         """
         Helper function to calculate the Petrosian "radius".
-        
+
         For the ellipse with semi-major axis ``a``, return the
         ratio of the mean flux over an elliptical annulus
         divided by the mean flux within the ellipse,
         minus "eta" (eq. 4 from Lotz et al. 2004). The root of
         this function is the Petrosian "radius".
-        
+
         """
         image = self._cutout_stamp_maskzeroed
 
@@ -1048,9 +1091,9 @@ class SourceMorphology(object):
         b_out = a_out / elongation
 
         ellip_annulus = photutils.EllipticalAnnulus(
-            center, a_in, a_out, b_out, theta)
+            center, a_in, a_out, b_out, theta=theta)
         ellip_aperture = photutils.EllipticalAperture(
-            center, a, b, theta)
+            center, a, b, theta=theta)
 
         # Force mean fluxes to be positive:
         ellip_annulus_mean_flux = np.abs(_aperture_mean_nomask(
@@ -1071,7 +1114,7 @@ class SourceMorphology(object):
         """
         Compute the Petrosian "radius" (actually the semi-major axis)
         for concentric elliptical apertures.
-        
+
         Notes
         -----
         The so-called "curve of growth" is not always monotonic,
@@ -1139,7 +1182,8 @@ class SourceMorphology(object):
         a = self._petro_extent_flux * self.rpetro_ellip
         b = a / self.elongation_asymmetry
         theta = self.orientation_asymmetry
-        ap = photutils.EllipticalAperture(self._asymmetry_center, a, b, theta)
+        ap = photutils.EllipticalAperture(
+            self._asymmetry_center, a, b, theta=theta)
         # Force flux sum to be positive:
         ap_sum = np.abs(ap.do_photometry(image, method='exact')[0][0])
         return ap_sum
@@ -1164,7 +1208,7 @@ class SourceMorphology(object):
         b_out = a_out / self.elongation_asymmetry
         theta = self.orientation_asymmetry
         ellip_annulus = photutils.EllipticalAnnulus(
-            (self._xc_stamp, self._yc_stamp), a_in, a_out, b_out, theta)
+            (self._xc_stamp, self._yc_stamp), a_in, a_out, b_out, theta=theta)
         ellip_annulus_mean_flux = _aperture_mean_nomask(
             ellip_annulus, cutout_smooth, method='exact')
 
@@ -1173,7 +1217,7 @@ class SourceMorphology(object):
         # Grow regions with 8-connected neighbor "footprint"
         s = ndi.generate_binary_structure(2, 2)
         labeled_array, num_features = ndi.label(above_threshold, structure=s)
-        
+
         # In some rare cases (e.g., Pan-STARRS J020218.5+672123_g.fits.gz),
         # this results in an empty segmap, so there is nothing to do.
         if num_features == 0:
@@ -1220,7 +1264,7 @@ class SourceMorphology(object):
 
             self.flag = 1
             return -99.0  # invalid
-        
+
         indices = np.arange(1, n+1)  # start at i=1
         gini = (np.sum((2*indices-n-1) * sorted_pixelvals) /
                 (float(n-1) * np.sum(sorted_pixelvals)))
@@ -1333,7 +1377,7 @@ class SourceMorphology(object):
     def _slice_skybox(self):
         """
         Try to find a region of the sky that only contains background.
-        
+
         In principle, a more accurate approach is possible
         (e.g. Shi et al. 2009, ApJ, 697, 1764).
         """
@@ -1342,7 +1386,7 @@ class SourceMorphology(object):
         mask = np.zeros(segmap.shape, dtype=np.bool8)
         if self._mask is not None:
             mask = self._mask[self._slice_stamp]
-        
+
         cur_skybox_size = self._skybox_size
         while True:
             for i in range(0, ny - cur_skybox_size):
@@ -1356,7 +1400,7 @@ class SourceMorphology(object):
             if cur_skybox_size <= 1:
                 warnings.warn('[skybox] Skybox not found.', AstropyUserWarning)
                 self.flag = 1
-                return (slice(0, 0), slice(0, 0))
+                return slice(0, 0), slice(0, 0)
             else:
                 cur_skybox_size //= 2
                 warnings.warn('[skybox] Reducing skybox size to %d.' % (
@@ -1441,7 +1485,7 @@ class SourceMorphology(object):
         """
         Helper function to determine the asymmetry and center of asymmetry.
         The idea is to minimize the output of this function.
-        
+
         Parameters
         ----------
         center : tuple or array-like
@@ -1451,7 +1495,7 @@ class SourceMorphology(object):
         kind : {'cas', 'outer', 'shape'}
             Whether to calculate the traditional CAS asymmetry (default),
             outer asymmetry or shape asymmetry.
-        
+
         Returns
         -------
         asym : The asymmetry statistic for the given center.
@@ -1494,7 +1538,7 @@ class SourceMorphology(object):
                               AstropyUserWarning)
                 self.flag = 1
                 return -99.0  # invalid
-            ap = photutils.EllipticalAnnulus(center, a_in, a_out, b_out, theta)
+            ap = photutils.EllipticalAnnulus(center, a_in, a_out, b_out, theta=theta)
         elif kind == 'shape':
             if np.isnan(self.rmax_circ) or (self.rmax_circ <= 0):
                 warnings.warn('[shape_asym] Invalid rmax_circ value.',
@@ -1524,7 +1568,7 @@ class SourceMorphology(object):
             else:
                 ap_area = _aperture_area(ap, mask_symmetric)
                 asym = (ap_abs_diff - ap_area*self._sky_asymmetry) / ap_abs_sum
-                
+
         return asym
 
     @lazyproperty
@@ -1536,7 +1580,7 @@ class SourceMorphology(object):
         center_0 = np.array([self._xc_stamp, self._yc_stamp])  # initial guess
         center_asym = opt.fmin(self._asymmetry_function, center_0,
                                args=(self._cutout_stamp_maskzeroed, 'cas'),
-                               xtol=1e-6, disp=0)
+                               xtol=1e-6, disp=False)
 
         # Check if flag was activated by _asymmetry_function:
         if self._use_centroid:
@@ -1621,7 +1665,7 @@ class SourceMorphology(object):
         image = self._cutout_stamp_maskzeroed
         asym = self._asymmetry_function(self._asymmetry_center,
                                         image, 'cas')
-        
+
         return asym
 
     def _radius_at_fraction_of_total_cas(self, fraction):
@@ -1632,16 +1676,16 @@ class SourceMorphology(object):
         image = self._cutout_stamp_maskzeroed
         center = self._asymmetry_center
         r_upper = self._petro_extent_cas * self.rpetro_circ
-        
+
         r, flag = _radius_at_fraction_of_total_circ(image, center, r_upper, fraction)
         self.flag = max(self.flag, flag)
-        
+
         if np.isnan(r) or (r <= 0.0):
             warnings.warn('[CAS] Invalid radius_at_fraction_of_total.',
                           AstropyUserWarning)
             self.flag = 1
             r = -99.0  # invalid
-        
+
         return r
 
     @lazyproperty
@@ -1677,7 +1721,7 @@ class SourceMorphology(object):
             C = -99.0  # invalid
         else:
             C = 5.0 * np.log10(self.r80 / self.r20)
-        
+
         return C
 
     @lazyproperty
@@ -1695,7 +1739,7 @@ class SourceMorphology(object):
 
         boxcar_size = int(self._petro_fraction_cas * self.rpetro_circ)
         image_smooth = ndi.uniform_filter(image, size=boxcar_size)
-        
+
         image_diff = image - image_smooth
         image_diff[image_diff < 0] = 0.0  # set negative pixels to zero
 
@@ -1711,7 +1755,10 @@ class SourceMorphology(object):
         if self._sky_smoothness == -99.0:  # invalid skybox
             S = ap_diff / ap_flux
         else:
-            S = (ap_diff - ap.area()*self._sky_smoothness) / ap_flux
+            if parse_version(photutils.__version__) <= parse_version('0.6'):
+                S = (ap_diff - ap.area()*self._sky_smoothness) / ap_flux
+            else:
+                S = (ap_diff - ap.area*self._sky_smoothness) / ap_flux
 
         if not np.isfinite(S):
             warnings.warn('Invalid smoothness.', AstropyUserWarning)
@@ -1750,13 +1797,13 @@ class SourceMorphology(object):
     def _segmap_mid_function(self, q):
         """
         Helper function to calculate the MID segmap.
-        
+
         For a given quantile ``q``, return the ratio of the mean flux of
         pixels at the level of ``q`` (within the main clump) divided by
         the mean of pixels above ``q`` (within the main clump).
         """
         locs_main_clump = self._segmap_mid_main_clump(q)
-        
+
         mean_flux_main_clump = np.mean(
             self._cutout_stamp_maskzeroed_no_bg_nonnegative[locs_main_clump])
         mean_flux_new_pixels = _quantile(
@@ -1776,7 +1823,7 @@ class SourceMorphology(object):
         """
         Create a new segmentation map as described in Section 4.3 from
         Freeman et al. (2013).
-        
+
         Notes
         -----
         This implementation improves upon previous ones by making
@@ -1784,7 +1831,7 @@ class SourceMorphology(object):
         used in the calculation, as well as other parameters.
         """
         num_pixelvals = len(self._sorted_pixelvals_stamp_no_bg_nonnegative)
-        
+
         # In some rare cases (as a consequence of an erroneous
         # initial segmap, as in J095553.0+694048_g.fits.gz),
         # the MID segmap is technically undefined because the
@@ -1796,7 +1843,7 @@ class SourceMorphology(object):
             warnings.warn('segmap_mid is undefined; using segmap_gini instead.',
                           AstropyUserWarning)
             return self._segmap_gini
-        
+
         # Find appropriate quantile using numerical solver
         q_min = 0.0
         q_max = 1.0
@@ -1885,7 +1932,7 @@ class SourceMorphology(object):
         """
         Calculate the multimode (M) statistic as described in
         Freeman et al. (2013) and Peth et al. (2016).
-        
+
         Notes
         -----
         In the original publication, Freeman et al. (2013)
@@ -1897,14 +1944,14 @@ class SourceMorphology(object):
         quantity (A2/A1)*A2 (as a function of the brightness
         threshold) but ultimately define the M statistic
         as the corresponding A2/A1 value.
-        
+
         The original IDL implementation only explores quantiles
         in the range [0.5, 1.0], at least with the default settings.
         While this might be useful in practice, in theory the
         maximum (A2/A1)*A2 value could also happen in the quantile
         range [0.0, 0.5], so here we take a safer, more general
         approach and search over [0.0, 1.0].
-        
+
         In practice, the quantity (A2/A1)*A2 is tricky to optimize.
         We improve over previous implementations by doing so
         in two stages: starting with a brute-force search
@@ -1912,7 +1959,7 @@ class SourceMorphology(object):
         original implementation, followed by a finer search using
         the basin-hopping method. This should do a better job of
         finding the global maximum.
-        
+
         """
         q_min = 0.0
         q_max = 1.0
@@ -1952,8 +1999,8 @@ class SourceMorphology(object):
         mid_bh_rel_temp = 0.5
 
         temperature = -1.0 * mid_bh_rel_temp * ratio_min
-        res = opt.basinhopping(self._multimode_ratio, q0,
-            minimizer_kwargs={"method": "Nelder-Mead"},
+        res = opt.basinhopping(
+            self._multimode_ratio, q0, minimizer_kwargs={"method": "Nelder-Mead"},
             niter=self._niter_bh_mid, T=temperature, stepsize=mid_stepsize,
             interval=self._niter_bh_mid/2, disp=False, seed=0)
         q_final = res.x[0]
@@ -1993,7 +2040,7 @@ class SourceMorphology(object):
         markers[ypeak, xpeak] = peak_labels
 
         mask = self._cutout_mid_smooth > 0
-        labeled_array = skimage.morphology.watershed(
+        labeled_array = skimage.segmentation.watershed(
             -self._cutout_mid_smooth, markers, connectivity=2, mask=mask)
 
         return labeled_array, peak_labels, xpeak, ypeak
@@ -2006,7 +2053,7 @@ class SourceMorphology(object):
         """
         labeled_array, peak_labels, xpeak, ypeak = self._watershed_mid
         num_peaks = len(peak_labels)
-        
+
         flux_sums = np.zeros(num_peaks, dtype=np.float64)
         for k, label in enumerate(peak_labels):
             locs = labeled_array == label
@@ -2015,7 +2062,7 @@ class SourceMorphology(object):
         sorted_flux_sums = flux_sums[sid]
         sorted_xpeak = xpeak[sid]
         sorted_ypeak = ypeak[sid]
-        
+
         return sorted_flux_sums, sorted_xpeak, sorted_ypeak
 
     @lazyproperty
@@ -2039,7 +2086,7 @@ class SourceMorphology(object):
         Peth et al. (2016).
         """
         image = np.float64(self._cutout_mid)  # skimage wants double
-        
+
         sorted_flux_sums, sorted_xpeak, sorted_ypeak = self._intensity_sums
         if len(sorted_flux_sums) == 0:
             warnings.warn('[deviation] There are no peaks.', AstropyUserWarning)
@@ -2048,7 +2095,7 @@ class SourceMorphology(object):
 
         xp = sorted_xpeak[0]
         yp = sorted_ypeak[0]
-        
+
         # Calculate centroid
         M = skimage.measure.moments(image, order=1)
         if M[0, 0] <= 0:
@@ -2089,7 +2136,7 @@ class SourceMorphology(object):
             r, flag = _radius_at_fraction_of_total_circ(
                 image, center, self.rmax_circ, 0.5)
             self.flag = max(self.flag, flag)
-        
+
         # In theory, this return value can also be NaN
         return r
 
@@ -2110,7 +2157,7 @@ class SourceMorphology(object):
                 image, center, self.elongation_asymmetry,
                 self.orientation_asymmetry, self.rmax_ellip, 0.5)
             self.flag = max(self.flag, flag)
-        
+
         # In theory, this return value can also be NaN
         return r
 
@@ -2119,7 +2166,7 @@ class SourceMorphology(object):
         """
         Construct a binary detection mask as described in Section 3.1
         from Pawlik et al. (2016).
-        
+
         Notes
         -----
         The original algorithm assumes that a circular aperture with
@@ -2145,7 +2192,11 @@ class SourceMorphology(object):
         circ_annulus = photutils.CircularAnnulus(center, r_in, r_out)
 
         # Convert circular annulus aperture to binary mask
-        circ_annulus_mask = circ_annulus.to_mask(method='center')[0]
+        if parse_version(photutils.__version__) <= parse_version('0.6'):
+            circ_annulus_mask = circ_annulus.to_mask(method='center')[0]
+        else:
+            circ_annulus_mask = circ_annulus.to_mask(method='center')
+
         # With the same shape as the postage stamp
         circ_annulus_mask = circ_annulus_mask.to_image((ny, nx))
         # Invert mask and exclude other sources
@@ -2197,7 +2248,7 @@ class SourceMorphology(object):
     def rmax_circ(self):
         """
         Return the distance (in pixels) from the pixel that minimizes
-        the asymmetry to the edge of the main source segment, similar 
+        the asymmetry to the edge of the main source segment, similar
         to Pawlik et al. (2016).
         """
         image = self._cutout_stamp_maskzeroed
@@ -2209,14 +2260,14 @@ class SourceMorphology(object):
         # Distances from all pixels to the center
         ypos, xpos = np.mgrid[0:ny, 0:nx]
         distances = np.sqrt((ypos-yc)**2 + (xpos-xc)**2)
-        
+
         # Only consider pixels within the segmap.
         rmax_circ = np.max(distances[self._segmap_shape_asym])
-        
+
         if rmax_circ == 0:
             warnings.warn('[rmax_circ] rmax_circ = 0!', AstropyUserWarning)
             self.flag = 1
-        
+
         return rmax_circ
 
     @lazyproperty
@@ -2242,11 +2293,11 @@ class SourceMorphology(object):
 
         # Only consider pixels within the segmap.
         rmax_ellip = np.max(r_ellip[self._segmap_shape_asym])
-        
+
         if rmax_ellip == 0:
             warnings.warn('[rmax_ellip] rmax_ellip = 0!', AstropyUserWarning)
             self.flag = 1
-        
+
         return rmax_ellip
 
     @lazyproperty
@@ -2257,7 +2308,7 @@ class SourceMorphology(object):
         """
         image = self._cutout_stamp_maskzeroed
         asym = self._asymmetry_function(self._asymmetry_center, image, 'outer')
-        
+
         return asym
 
     @lazyproperty
@@ -2296,7 +2347,7 @@ class SourceMorphology(object):
             a_in = self.rhalf_ellip
         b_out = a_out / self.elongation_asymmetry
         ellip_annulus = photutils.EllipticalAnnulus(
-            center, a_in, a_out, b_out, theta)
+            center, a_in, a_out, b_out, theta=theta)
         ellip_annulus_mean_flux = _aperture_mean_nomask(
             ellip_annulus, image, method='exact')
         if ellip_annulus_mean_flux <= 0.0:
@@ -2422,14 +2473,14 @@ def source_morphology(image, segmap, **kwargs):
     """
     Calculate the morphological parameters of all sources in ``image``
     as labeled by ``segmap``.
-    
+
     Parameters
     ----------
     image : array-like
         A 2D image containing the sources of interest.
         The image must already be background-subtracted.
     segmap : array-like (int) or `photutils.SegmentationImage`
-        A 2D segmentation map where different sources are 
+        A 2D segmentation map where different sources are
         labeled with different positive integer values.
         A value of zero is reserved for the background.
 
@@ -2447,11 +2498,11 @@ def source_morphology(image, segmap, **kwargs):
     See Also
     --------
     `SourceMorphology` : Class to measure morphological parameters.
-    
+
     Examples
     --------
     See `README.rst` for usage examples.
-    
+
     References
     ----------
     See `README.rst` for a list of references.
@@ -2463,6 +2514,6 @@ def source_morphology(image, segmap, **kwargs):
     sources_morph = []
     for label in segmap.labels:
         sources_morph.append(SourceMorphology(image, segmap, label, **kwargs))
-        print('Finished processing source %d.\n' % (label))
+        print('Finished processing source %d.\n' % (label,))
 
     return sources_morph
